using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.IO;
using System.Net;
using System.Threading;

namespace MGS
{
    public partial class MGSService : ServiceBase
    {
        public MGSService()
        {
            InitializeComponent();
        }

        private string currentMgStatus = "";
        private string logFilePath = "";
        private int queryTime = 100000;
        private int startTime = 100000;
        private string url = "";
        private int maxReqTime = 0;
        private int dailyRestart = 0;
        private Thread thread;

        private bool testSize = false;
        private bool usePsKill = false;
        private string psKillPath = "";

        //2009-07-02
        private string mgDisplayName = "";
        private string mgProcessName = "";

        public void StartDebug()
        {
            OnStart(null);
        }

        private void KillAndRestart()
        {
            ServiceController MgService = null;

            //Combien de fois on attend avant de forcer un kill
            int maxRetries = 0;

            string currentStatus = "";

            //Console.WriteLine("KILL! AND RESTART!");
            currentMgStatus = currentMgStatus + "\r\n" + "Killing " + mgProcessName + " and restarting!";

            if (usePsKill)
            {
                PsKillMG();
                Thread.Sleep(1000);
            }
            else
            {
                //let's kill mapguide
                Process[] ProcArray = Process.GetProcessesByName(mgProcessName);
                if (ProcArray.Length > 0)
                    foreach (Process ProcessFound in ProcArray)
                    {
                        if (!ProcessFound.HasExited)
                        {
                            ProcessFound.Kill();
                            ProcessFound.WaitForExit(1000);
                        }
                    }
            }

            //let's get the service
            MgService = GetMapGuideService();

            currentStatus = MgService.Status.ToString();

            if (currentStatus == "Running")
            {
                MgService.Stop();
                do
                {
                    Thread.Sleep(1000);
                    MgService = GetMapGuideService();
                    currentStatus = MgService.Status.ToString();
                    //Console.WriteLine(currentStatus);
                    currentMgStatus = currentMgStatus + "\r\n" + currentStatus;
                    //10 tries it gives him 10 seconds to stop
                    maxRetries = maxRetries + 1;
                } while (currentStatus != "Stopped" || maxRetries != 10);

                if (maxRetries == 10)
                {
                    if (usePsKill)
                    {
                        PsKillMG();
                        Thread.Sleep(1000);
                    }
                    else
                    {
                        //let's kill mapguide
                        Process[] ProcArray2 = Process.GetProcessesByName(mgProcessName);
                        if (ProcArray2.Length > 0)
                            foreach (Process ProcessFound in ProcArray2)
                            {
                                if (!ProcessFound.HasExited)
                                {
                                    ProcessFound.Kill();
                                    ProcessFound.WaitForExit(1000);
                                }
                            }
                    }
                }
            }

            if (currentStatus == "Stopped")
            {
                MgService.Start();
                do
                {
                    Thread.Sleep(1000);
                    MgService = GetMapGuideService();
                    currentStatus = MgService.Status.ToString();
                    //Console.WriteLine(currentStatus);
                    currentMgStatus = currentMgStatus + "\r\n" + currentStatus;
                } while (currentStatus != "Running");

                //now started, let's give him a chance by waiting a little
                currentMgStatus = currentMgStatus + "\r\n" + "Waiting for MapGuide to warm up";
                Thread.Sleep(startTime);
                currentMgStatus = currentMgStatus + "\r\n" + "MapGuide should be running";
            }
            
        }

        private StringBuilder QueryMapagent(StringBuilder sb, ref long reqsize)
        {
            // used on each read operation
            byte[] buf = new byte[8192];

            HttpWebRequest reqImage = (HttpWebRequest)WebRequest.Create(url);

            try
            {
                //On lui donne un maximum de 60000 ms = 1 min sinon il est en erreur
                reqImage.Timeout = maxReqTime;

                HttpWebResponse mapagentResponse = (HttpWebResponse)reqImage.GetResponse();
                
                Stream resStream = mapagentResponse.GetResponseStream();

                reqsize = mapagentResponse.ContentLength;

                int count = 0;

                
                do
                {
                    // fill the buffer with data
                    count = resStream.Read(buf, 0, buf.Length);

                    // make sure we read some data
                    if (count != 0)
                    {
                        sb.Append(buf);
                    }

                }
                while (count > 0); // any more data to read?
            }
            catch (Exception)
            {
                sb.Append("Error");
            }

            return (sb);
        }

        private ServiceController GetMapGuideService()
        {
            ServiceController[] services = ServiceController.GetServices();
            ServiceController MgService = null;

            foreach (ServiceController service in services)
            {
                if (service.DisplayName == mgDisplayName)
                {
                    MgService = service;
                }
            }

            return (MgService);
        }

        private void ReadConfig()
        {
            //change this
            string[] config = File.ReadAllLines("c:\\mgstatus\\config.ini");

            logFilePath = config[1];
            queryTime = int.Parse(config[3]) * 1000;
            startTime = int.Parse(config[5]) * 1000;
            url = config[7];
            maxReqTime = int.Parse(config[9]) * 1000;

            if (config[11] == "true")
                testSize = true;
            else
                testSize = false;

            dailyRestart = int.Parse(config[13]);

            if (config[15] == "true")
            {
                usePsKill = true;
                psKillPath = config[17];
            }
            else
                usePsKill = false;

            //2009-07-02
            mgDisplayName = config[19];

            mgProcessName = config[21];
        }

        protected override void OnStart(string[] args)
        {
            thread = new System.Threading.Thread(new System.Threading.ParameterizedThreadStart(this.Run));
            thread.Name = "MgStatus";
            thread.Start(this);
        }

        private void PsKillMG()
        {
            currentMgStatus = currentMgStatus + "\r\n" + "Using pskill on MapGuide!";

            Process p = new Process();
            p.StartInfo.FileName = psKillPath;
 
            p.StartInfo.UseShellExecute = false;  
            p.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;  
            p.StartInfo.CreateNoWindow = true;

            p.StartInfo.Arguments = mgProcessName;

            p.Start();

            p.WaitForExit();

            p.Close();
        }
        
        private void Run(object data)
        {

            ServiceController MgService = null;
            //let's get the service
            MgService = GetMapGuideService();

            Boolean firstFetch = true;
            long refenceSize = 0;
            long currentSize = 0;

            string timeForLog = "";

            int todayInt = 0;
            int lastDailyRestartDay = 0;

            ReadConfig();

            if (File.Exists(logFilePath) == false)
            {
                File.WriteAllText(logFilePath, "");
            }

            do
            {       
                //let's get the service
                MgService = GetMapGuideService();

                //clear Status
                currentMgStatus = "";

                //current time
                timeForLog = "\r\n***" + System.DateTime.Now.Year.ToString() + "-" + System.DateTime.Now.Month.ToString();
                timeForLog = timeForLog + "-" + System.DateTime.Now.Day.ToString() + " " + System.DateTime.Now.Hour.ToString();
                timeForLog = timeForLog + ":" + System.DateTime.Now.Minute.ToString() + ":" + System.DateTime.Now.Second.ToString() + "***\r\n";


                if (MgService == null)
                {
                    //Console.WriteLine("The MapGuide Service was not found");
                    currentMgStatus = currentMgStatus + "\r\n" + "The MapGuide Service was not found";
                }
                else
                {
                    //Console.WriteLine("MapGuide Service found");
                    currentMgStatus = currentMgStatus + "\r\n" + "MapGuide Service found";
                    //Console.WriteLine(MgService.Status.ToString());
                    currentMgStatus = currentMgStatus + "\r\n" + MgService.Status.ToString();

                    //Time to restart!!
                    if (System.DateTime.Now.Hour == dailyRestart)
                    {
                        todayInt = System.DateTime.Now.Day;

                        if (todayInt != lastDailyRestartDay)
                        {
                            currentMgStatus = currentMgStatus + "\r\n" + "Initiating MapGuide's daily restart";
                            KillAndRestart();
                            File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
                            lastDailyRestartDay = todayInt;
                            continue;
                        }
                    }

                    if (MgService.Status.ToString() == "Stopped")
                    {
                        KillAndRestart();
                        File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
                        continue;
                    }
                    else
                    {
                        //on lui parle
                        StringBuilder sb = new StringBuilder();


                        sb = QueryMapagent(sb, ref currentSize);

                        //Did we get something?
                        if (currentSize > 0)
                        {
                            if (sb.ToString() == "Error")
                            {
                                currentMgStatus = currentMgStatus + "\r\n" + "Something was wrong while querying MapGuide!";
                                KillAndRestart();
                                File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
                                continue;
                            }
                            else
                                //Console.WriteLine("I got something");
                                currentMgStatus = currentMgStatus + "\r\n" + "I got something";

                            //Reference size
                            if (firstFetch)
                            {
                                firstFetch = false;
                                refenceSize = currentSize;
                            }

                            if (testSize)
                            {
                                //Check if it's size is still the same
                                if (currentSize == refenceSize)
                                    //Console.WriteLine("It was the correct size:" + currentSize.ToString());
                                    currentMgStatus = currentMgStatus + "\r\n" + "It was the correct size:" + currentSize.ToString();
                                else
                                {
                                    //Console.WriteLine("It wasn't the correct size! " + currentSize.ToString() + " , i'll try again!");
                                    currentMgStatus = currentMgStatus + "\r\n" + "It wasn't the correct size! " + currentSize.ToString() + " , i'll try again!";

                                    KillAndRestart();
                                    File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
                                    continue;
                                }
                            }

                        }
                        else
                        {
                            currentMgStatus = currentMgStatus + "\r\n" + "I got nothing";

                            KillAndRestart();
                            File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
                            continue;
                        }

                    }
                }

            //write status
            File.AppendAllText(logFilePath, timeForLog + currentMgStatus);
            //tryagain later
            Thread.Sleep(queryTime);
            }
            while (true);

        }
        //http://192.168.1.21/mapguide/mapagent/mapagent.fcgi?OPERATION=GETMAPIMAGE&VERSION=1.0.0&MAPNAME=geobase&LOCALE=en&MAPDEFINITION=Library%3A%2F%2Fgeobase.MapDefinition&FORMAT=JPG&SETVIEWCENTERX=307545&SETVIEWCENTERY=5036491&SETVIEWSCALE=5000&SETDISPLAYDPI=96&SETDISPLAYWIDTH=300&SETDISPLAYHEIGHT=200&USERNAME=Anonymous&PASSWORD=
        protected override void OnStop()
        {
            //let's kill the service
            Process[] ProcArray3 = Process.GetProcessesByName("MGS.exe");
            if (ProcArray3.Length > 0)
                foreach (Process ProcessFound in ProcArray3)
                {
                    if (!ProcessFound.HasExited)
                    {
                        ProcessFound.Kill();
                        ProcessFound.WaitForExit(1000);
                    }
                }

        }
    }
}
